#pragma once

// Export section
#if defined(__has_include)
#    if __has_include(<UpdateManagerExport.hpp>)
#        include <UpdateManagerExport.hpp>
#    else
#        define UpdateManagerExport
#    endif
#elif defined(UpdateManager_Native_EXPORTS)
#    include <UpdateManagerExport.hpp>
#else
#    define UpdateManagerExport
#endif

#include <cstddef>   // size_t

namespace LaCie
{
// Forward declaration
class IUpdateTask;
enum class TaskStatus;
enum class TaskResult;

class UpdateManagerExport UpdateManager final
{
public:
    UpdateManager();
    UpdateManager(const UpdateManager &other) = delete;
    UpdateManager &operator=(const UpdateManager &other) = delete;
    UpdateManager(UpdateManager &&t) noexcept;
    UpdateManager &operator=(UpdateManager &&t) noexcept;
    ~UpdateManager();

    /*!
     * Get JSON representation of all KNOWN connected chips with general and specific information.
     * Function is non-thread safely. Use external mutex locker!
     * By default - empty json array will be returned.
     * \param[in]  updateJsonStr Update JSON will be used as additional vid/pid pair configurator.
     * \return JSON array
    */
    const char *fillJSON(const char *updateJsonStr = nullptr) const;

    /*!
    * Create new device updating task via preconfigured JSON data
     * Function is thread safely.
    * \param[in]  inputJson Full description of all general and specific data fields.
    * \return Task pointer if JSON data is valid and task successfully started, otherwise - nullptr.
     * Task will be detached from internal storage. Potentially dangerous, but
     * useful to wrap raw pointer into smart pointer (own life time control).
     * It is necessary to manually start/stop task.
    */
    IUpdateTask *makeNewUpdateTask(const char *inputJson) const;

    /*!
     * Get all KNOWN chips information as string representation (Description, VID, PID)
     * Function is non-thread safely. Use external mutex locker!
     * \param[out]  count Found device list count.
     * \return strings array
    */
    static const char **supportedDeviceList(size_t &count);

protected:
    class Data;
    Data *d{nullptr};
};
}   // namespace LaCie
